import auth from "@/plugins/auth";
import router, { constantRoutes, dynamicRoutes } from "@/router";
import { getRouters } from "@/api/menu";
import Layout from "@/layout/index";
import ParentView from "@/components/ParentView";
import InnerLink from "@/layout/components/InnerLink";

// 匹配views里面所有的.vue文件
const modules = import.meta.glob("./../../views/**/*.vue");

const usePermissionStore = defineStore("permission", {
  state: () => ({
    routes: [],
    addRoutes: [],
    defaultRoutes: [],
    topbarRouters: [],
    sidebarRouters: [],
  }),
  actions: {
    setRoutes(routes) {
      this.addRoutes = routes;
      this.routes = constantRoutes.concat(routes);
    },
    setDefaultRoutes(routes) {
      this.defaultRoutes = constantRoutes.concat(routes);
    },
    setTopbarRoutes(routes) {
      this.topbarRouters = routes;
    },
    setSidebarRouters(routes) {
      this.sidebarRouters = routes;
    },
    generateRoutes(roles) {
      return new Promise((resolve) => {
        // 向后端请求路由数据
        // getRouters().then(res => {
        //   const sdata = JSON.parse(JSON.stringify(res.data))
        //   const rdata = JSON.parse(JSON.stringify(res.data))
        //   const defaultData = JSON.parse(JSON.stringify(res.data))
        //   const sidebarRoutes = filterAsyncRouter(sdata)
        //   const rewriteRoutes = filterAsyncRouter(rdata, false, true)
        //   const defaultRoutes = filterAsyncRouter(defaultData)
        //   const asyncRoutes = filterDynamicRoutes(dynamicRoutes)
        //   asyncRoutes.forEach(route => { router.addRoute(route) })
        //   this.setRoutes(rewriteRoutes)
        //   this.setSidebarRouters(constantRoutes.concat(sidebarRoutes))
        //   this.setDefaultRoutes(sidebarRoutes)
        //   this.setTopbarRoutes(defaultRoutes)
        //   resolve(rewriteRoutes)
        // })

        // 改为固定菜单数据演示用
        const res = {
          code: 200,
          msg: "success",
          data: [
            {
              alwaysShow: true,
              path: "/system",
              component: "Layout",
              redirect: "noRedirect",
              name: "System",
              hidden: false,
              meta: {
                icon: "system",
                link: null,
                noCache: false,
                title: "系统管理",
              },
              children: [
                {
                  component: "system/user/index",
                  hidden: false,
                  meta: {
                    title: "用户管理",
                    icon: "user",
                    noCache: false,
                    link: null,
                  },
                  name: "User",
                  path: "user",
                },
                {
                  component: "system/role/index",
                  hidden: false,
                  meta: {
                    title: "角色管理",
                    icon: "peoples",
                    noCache: false,
                    link: null,
                  },
                  name: "Role",
                  path: "role",
                },
                {
                  component: "system/menu/index",
                  hidden: false,
                  meta: {
                    title: "菜单管理",
                    icon: "tree-table",
                    noCache: false,
                    link: null,
                  },
                  name: "Menu",
                  path: "menu",
                },
                {
                  component: "system/dept/index",
                  hidden: false,
                  meta: {
                    title: "部门管理",
                    icon: "tree",
                    noCache: false,
                    link: null,
                  },
                  name: "Dept",
                  path: "dept",
                },
                {
                  component: "system/notice/index",
                  hidden: false,
                  meta: {
                    title: "通知公告",
                    icon: "message",
                    noCache: false,
                    link: null,
                  },
                  name: "Notice",
                  path: "notice",
                },
                {
                  component: "ParentView",
                  hidden: false,
                  meta: {
                    title: "日志管理",
                    icon: "log",
                    noCache: false,
                    link: null,
                  },
                  name: "Log",
                  path: "log",
                  children: [
                    {
                      component: "monitor/operlog/index",
                      hidden: false,
                      meta: {
                        title: "操作日志",
                        icon: "form",
                        noCache: false,
                        link: null,
                      },
                      name: "Operlog",
                      path: "operlog",
                    },
                    {
                      component: "monitor/logininfor/index",
                      hidden: false,
                      meta: {
                        title: "登录日志",
                        icon: "logininfor",
                        noCache: false,
                        link: null,
                      },
                      name: "Logininfor",
                      path: "logininfor",
                    },
                  ],
                },
              ],
            },
          ],
        };
    
        const sdata = JSON.parse(JSON.stringify(res.data));
        const rdata = JSON.parse(JSON.stringify(res.data));
        const defaultData = JSON.parse(JSON.stringify(res.data));
        const sidebarRoutes = filterAsyncRouter(sdata);
        const rewriteRoutes = filterAsyncRouter(rdata, false, true);
        const defaultRoutes = filterAsyncRouter(defaultData);
        const asyncRoutes = filterDynamicRoutes(dynamicRoutes);
        asyncRoutes.forEach((route) => {
          router.addRoute(route);
        });
        this.setRoutes(rewriteRoutes);
        this.setSidebarRouters(constantRoutes.concat(sidebarRoutes));
        this.setDefaultRoutes(sidebarRoutes);
        this.setTopbarRoutes(defaultRoutes);
        resolve(rewriteRoutes);
      });
    },
  },
});

// 遍历后台传来的路由字符串，转换为组件对象
function filterAsyncRouter(asyncRouterMap, lastRouter = false, type = false) {
  return asyncRouterMap.filter((route) => {
    if (type && route.children) {
      route.children = filterChildren(route.children);
    }
    if (route.component) {
      // Layout ParentView 组件特殊处理
      if (route.component === "Layout") {
        route.component = Layout;
      } else if (route.component === "ParentView") {
        route.component = ParentView;
      } else if (route.component === "InnerLink") {
        route.component = InnerLink;
      } else {
        route.component = loadView(route.component);
      }
    }
    if (route.children != null && route.children && route.children.length) {
      route.children = filterAsyncRouter(route.children, route, type);
    } else {
      delete route["children"];
      delete route["redirect"];
    }
    return true;
  });
}

function filterChildren(childrenMap, lastRouter = false) {
  var children = [];
  childrenMap.forEach((el) => {
    el.path = lastRouter ? lastRouter.path + "/" + el.path : el.path;
    if (el.children && el.children.length && el.component === "ParentView") {
      children = children.concat(filterChildren(el.children, el));
    } else {
      children.push(el);
    }
  });
  return children;
}

// 动态路由遍历，验证是否具备权限
export function filterDynamicRoutes(routes) {
  const res = [];
  routes.forEach((route) => {
    if (route.permissions) {
      if (auth.hasPermiOr(route.permissions)) {
        res.push(route);
      }
    } else if (route.roles) {
      if (auth.hasRoleOr(route.roles)) {
        res.push(route);
      }
    }
  });
  return res;
}

export const loadView = (view) => {
  let res;
  for (const path in modules) {
    const dir = path.split("views/")[1].split(".vue")[0];
    if (dir === view) {
      res = () => modules[path]();
    }
  }
  return res;
};

export default usePermissionStore;
